#Region "Copyright and License"
'
' DbTom ReportingService Module
' Copyright (c) 2005
' by Thomas Trefz (tt@dbresource.de)
'
' Download latest version at http://www.dbresource.de
' No registration required for download
'
' Parts of this module based on an article from Marian Dumitrascu at codeproject:
' http://www.codeproject.com/useritems/SQLRSViewer.asp
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
#End Region

Imports System.IO
Imports DotNetNuke
Imports DotNetNuke.Common
Imports DotNetNuke.Services.Exceptions
Imports DotNetNuke.Services.Localization
Imports DotNetNuke.Common.Utilities
Imports ReportService

Namespace DbTom.ReportingService

    ''' -----------------------------------------------------------------------------
    ''' Namespace:  DbTom.ReportingService
    ''' Project:    DbTom Module Pack for DotNetNuke
    ''' Class:      ReportViewer
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The ReportViewer form is used to show a report incl. parameters
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    '''     [ttr]   20.02.2005  created
    '''     [ttr]   21.02.2005  integrate module settings for reportname,
    '''                         username, password and domain
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public MustInherit Class ReportViewer

        Inherits Entities.Modules.PortalModuleBase

#Region "Controls"

        ' form controls
        Protected WithEvents TablePrompts As System.Web.UI.HtmlControls.HtmlTable
        Protected WithEvents ReportPlaceholder As System.Web.UI.HtmlControls.HtmlGenericControl
        Protected WithEvents txtReport As System.Web.UI.HtmlControls.HtmlInputHidden

#End Region

#Region "Private Members"

        Private m_reportServerURL As String = ""
        Private m_reportPath As String = ""
        Private m_reportUser As String = ""
        Private m_reportUserPwd As String = ""
        Private m_reportDomain As String = ""
        Private m_reportFormat As String = "HTML4.0"
        Private m_reportZoom As String = "100"
        Private m_Dependencies As New System.Collections.Hashtable
        Private m_canRendered As Boolean = True

#End Region

#Region "Public Properties"

        Public Property ReportServerURL() As String
            Get
                If m_reportServerURL <> "" Then
                    ReportServerURL = m_reportServerURL
                Else
                    ReportServerURL = CType(Me.Settings("reportserverurl"), String)
                End If
            End Get
            Set(ByVal Value As String)
                m_reportServerURL = Value
            End Set
        End Property

        Public Property ReportPath() As String
            Get
                If m_reportPath <> "" Then
                    ReportPath = m_reportPath
                Else
                    ReportPath = CType(Me.Settings("reportpath"), String)
                End If
            End Get
            Set(ByVal Value As String)
                m_reportPath = Value
            End Set
        End Property

        Public Property ReportDomain() As String
            Get
                If m_reportDomain <> "" Then
                    ReportDomain = m_reportDomain
                Else
                    ReportDomain = CType(Me.Settings("reportuserdomain"), String)
                End If
            End Get
            Set(ByVal Value As String)
                m_reportDomain = Value
            End Set
        End Property

        Public Property ReportUser() As String
            Get
                If m_reportUser <> "" Then
                    ReportUser = m_reportUser
                Else
                    ReportUser = CType(Me.Settings("reportusername"), String)
                End If
            End Get
            Set(ByVal Value As String)
                m_reportUser = Value
            End Set
        End Property

        Public Property ReportUserPwd() As String
            Get
                If m_reportUserPwd <> "" Then
                    ReportUserPwd = m_reportDomain
                Else
                    ReportUserPwd = CType(Me.Settings("reportuserpassword"), String)
                End If
            End Get
            Set(ByVal Value As String)
                m_reportUserPwd = Value
            End Set
        End Property

        Public Property ReportZoom() As String
            Get
                ReportZoom = m_reportZoom
            End Get
            Set(ByVal Value As String)
                m_reportZoom = Value
            End Set
        End Property

        Public Property ReportFormat() As String
            Get
                ReportFormat = m_reportFormat
            End Get
            Set(ByVal Value As String)
                m_reportFormat = Value
            End Set
        End Property

#End Region

#Region "Private Methods"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' GetDropDownZoom creates the html select control on the fly
        ''' </summary>
        ''' <returns>
        '''     System.Web.UI.HtmlControls.HtmlSelect
        ''' </returns>
        ''' <parameters>
        '''     <zoom>
        '''         Current value for zoom to select in the html select control
        '''     </zoom>
        '''     <CssClass>
        '''         CSS Class name to be used by the html select control. This
        '''         Parameter is optional. Default value is NormalTextBox.
        '''     </CssClass>
        ''' </parameters>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''     [ttr]   20.02.2005  created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Function GetDropDownZoom(ByVal Zoom As String, Optional ByVal CssCLass As String = "NormalTextBox") As System.Web.UI.HtmlControls.HtmlSelect

            ' create new html select list instance, set id and name
            Dim objSelect As New System.Web.UI.HtmlControls.HtmlSelect
            objSelect.ID = "Zoom"
            objSelect.Name = "Zoom"
            objSelect.Attributes.Add("Class", CssCLass)

            ' create var to hold instance of type ListItem
            Dim objListItem As System.Web.UI.WebControls.ListItem

            ' add static items to the list
            objListItem = New System.Web.UI.WebControls.ListItem("Page Width", "Page Width")
            If Zoom = objListItem.Value Then objListItem.Selected = True
            objSelect.Items.Add(objListItem)

            objListItem = New System.Web.UI.WebControls.ListItem("200%", "200")
            If Zoom = objListItem.Value Then objListItem.Selected = True
            objSelect.Items.Add(objListItem)

            objListItem = New System.Web.UI.WebControls.ListItem("150%", "150")
            If Zoom = objListItem.Value Then objListItem.Selected = True
            objSelect.Items.Add(objListItem)

            objListItem = New System.Web.UI.WebControls.ListItem("100%", "100")
            If Zoom = objListItem.Value Then objListItem.Selected = True
            objSelect.Items.Add(objListItem)

            objListItem = New System.Web.UI.WebControls.ListItem("75%", "75")
            If Zoom = objListItem.Value Then objListItem.Selected = True
            objSelect.Items.Add(objListItem)

            objListItem = New System.Web.UI.WebControls.ListItem("50%", "50")
            If Zoom = objListItem.Value Then objListItem.Selected = True
            objSelect.Items.Add(objListItem)

            objListItem = New System.Web.UI.WebControls.ListItem("25%", "25")
            If Zoom = objListItem.Value Then objListItem.Selected = True
            objSelect.Items.Add(objListItem)

            Return objSelect

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' GetDropDownFormat creates the html select control on the fly
        ''' </summary>
        ''' <returns>
        '''     System.Web.UI.HtmlControls.HtmlSelect
        ''' </returns>
        ''' <parameters>
        '''     <format>
        '''         Current value for format to select in the html select control
        '''     </format>
        '''     <CssClass>
        '''         CSS Class name to be used by the html select control. This
        '''         Parameter is optional. Default value is NormalTextBox.
        '''     </CssClass>
        ''' </parameters>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''     [ttr]   20.02.2005  created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Function GetDropDownFormat(ByVal Format As String, Optional ByVal CssCLass As String = "NormalTextBox") As System.Web.UI.HtmlControls.HtmlSelect

            ' create new html select list instance, set id and name
            Dim objSelect As New System.Web.UI.HtmlControls.HtmlSelect
            objSelect.ID = "Format"
            objSelect.Name = "Format"
            objSelect.Attributes.Add("Class", CssCLass)

            ' create var to hold instance of type ListItem
            Dim objListItem As System.Web.UI.WebControls.ListItem

            objListItem = New System.Web.UI.WebControls.ListItem("HTML", "HTML4.0")
            If Format = objListItem.Value Then objListItem.Selected = True
            objSelect.Items.Add(objListItem)

            objListItem = New System.Web.UI.WebControls.ListItem("Acrobat (PDF) file", "PDF")
            If Format = objListItem.Value Then objListItem.Selected = True
            objSelect.Items.Add(objListItem)

            objListItem = New System.Web.UI.WebControls.ListItem("Excel", "EXCEL")
            If Format = objListItem.Value Then objListItem.Selected = True
            objSelect.Items.Add(objListItem)

            objListItem = New System.Web.UI.WebControls.ListItem("TIFF file", "IMAGE")
            If Format = objListItem.Value Then objListItem.Selected = True
            objSelect.Items.Add(objListItem)

            objListItem = New System.Web.UI.WebControls.ListItem("Web archive", "MHTML")
            If Format = objListItem.Value Then objListItem.Selected = True
            objSelect.Items.Add(objListItem)

            objListItem = New System.Web.UI.WebControls.ListItem("CSV (comma delimited)", "CSV")
            If Format = objListItem.Value Then objListItem.Selected = True
            objSelect.Items.Add(objListItem)

            objListItem = New System.Web.UI.WebControls.ListItem("XML file with report data", "XML")
            If Format = objListItem.Value Then objListItem.Selected = True
            objSelect.Items.Add(objListItem)

            Return objSelect

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' BuildParameterTable creates the html table holding the controls to select
        ''' individual values for report execution.
        ''' </summary>
        ''' <parameters>
        '''     <ParamterArray>
        '''         Array wth all report parameters and their values
        '''     </ParameterArray>
        '''     <HTMLTable>
        '''         Referenz to the html table control to use for parameter controls
        '''     </HTMLTable>
        ''' </parameters>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''     [ttr]   21.02.2005  created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub BuildParameterTable(ByVal ParameterArray As ReportParameter(), ByVal HTMLTable As System.Web.UI.HtmlControls.HtmlTable)

            Try

                Dim i As Integer = 0
                Dim promptType As String
                Dim TR As System.Web.UI.HtmlControls.HtmlTableRow
                Dim TD As System.Web.UI.HtmlControls.HtmlTableCell

                For Each parameter As ReportParameter In ParameterArray

                    ' if any of parameters doesn't have valid  values the report cannot be rendered
                    If parameter.State <> ParameterStateEnum.HasValidValue Then
                        m_canRendered = False
                    End If
                    If parameter.DefaultValues(0) = "" And parameter.AllowBlankSpecified And Not parameter.AllowBlank Then
                        m_canRendered = False
                    End If
                    If parameter.DefaultValues(0) Is Nothing And parameter.NullableSpecified And Not parameter.Nullable Then
                        m_canRendered = False
                    End If

                    If i Mod 2 = 0 Then
                        TR = New System.Web.UI.HtmlControls.HtmlTableRow
                    End If

                    'set the propmt text
                    TD = New System.Web.UI.HtmlControls.HtmlTableCell
                    TD.InnerHtml = parameter.Prompt
                    TD.Attributes.Add("Class", "SubHead")
                    TR.Controls.Add(TD)

                    'set the prompt input control
                    TD = New System.Web.UI.HtmlControls.HtmlTableCell

                    promptType = "textbox"
                    If parameter.ValidValuesQueryBased Then
                        promptType = "dropdown"
                    End If

                    If Not parameter.ValidValues Is Nothing Then
                        If parameter.ValidValues.Length > 1 Then
                            promptType = "dropdown"
                        End If
                    End If

                    Select Case promptType

                        Case "dropdown"

                            ' create html select control with valid values and add to html column
                            TD.Controls.Add(BuildParameterDropDown(parameter))

                        Case "textbox"

                            ' create html input control with valid value and add to html column
                            TD.Controls.Add(BuildParameterTextBox(parameter))

                    End Select

                    TR.Controls.Add(TD)

                    'add this row to the table 
                    If i Mod 2 = 1 Then
                        HTMLTable.Controls.Add(TR)
                    End If

                    i += 1

                Next 'from: For Each Parameter As ReportParameter In Parameters

                If i Mod 2 = 1 Then
                    HTMLTable.Controls.Add(TR)
                End If

            Catch ex As Exception
                m_canRendered = False
            End Try

        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' BuildParameterDropDown creates the html select control for one parameter
        ''' </summary>
        ''' <returns>
        '''     System.Web.UI.HtmlControls.HtmlSelect
        ''' </returns>
        ''' <parameters>
        '''     <Parameter>
        '''         Report Parameter to create html select control
        '''     </Parameter>
        ''' </parameters>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''     [ttr]   21.02.2005  created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Function BuildParameterDropDown(ByVal Parameter As ReportParameter) As System.Web.UI.HtmlControls.HtmlSelect

            Dim objSelect As New System.Web.UI.HtmlControls.HtmlSelect
            Dim objListItem As System.Web.UI.WebControls.ListItem

            objSelect.ID = Parameter.Name
            objSelect.Name = Parameter.Name
            objSelect.Attributes.Add("Class", "NormalTextBox")

            If Not Parameter.ValidValues Is Nothing And Parameter.ValidValues.Length > 0 Then
                If Parameter.State = ParameterStateEnum.MissingValidValue Or Parameter.State = ParameterStateEnum.HasOutstandingDependencies Then
                    m_canRendered = False
                    objListItem = New System.Web.UI.WebControls.ListItem("<Select a Value>", "")
                    objSelect.Items.Add(objListItem)
                End If

                For Each vv As ValidValue In Parameter.ValidValues

                    objListItem = New System.Web.UI.WebControls.ListItem(vv.Label, vv.Value)
                    'see if this value is the same with the default value 
                    'in which case we make the current list item selected
                    If vv.Value = Parameter.DefaultValues(0) And Parameter.State = ParameterStateEnum.HasValidValue Then
                        objListItem.Selected = True
                    End If
                    objSelect.Items.Add(objListItem)

                Next

            End If

            'check if this parameter have dependencies.
            'If it does then we need to reload the page to reload the value of dependent params
            If m_Dependencies.Contains(Parameter.Name) Then
                objSelect.Attributes.Add("OnChange", "this.form.submit();return true")
            End If

            Return objSelect

        End Function

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' BuildParameterTextBox creates the html input control for one parameter
        ''' </summary>
        ''' <returns>
        '''     System.Web.UI.HtmlControls.HtmlInput
        ''' </returns>
        ''' <parameters>
        '''     <Parameter>
        '''         Report Parameter to create html input control
        '''     </Parameter>
        ''' </parameters>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''     [ttr]   21.02.2005  created
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Function BuildParameterTextBox(ByVal Parameter As ReportParameter) As System.Web.UI.HtmlControls.HtmlInputText

            Dim objTextBox As New System.Web.UI.HtmlControls.HtmlInputText
            objTextBox.ID = Parameter.Name
            objTextBox.Name = Parameter.Name
            objTextBox.Attributes.Add("Class", "NormalTextBox")

            If Parameter.DefaultValues.Length > 0 Then
                If Parameter.Type = ParameterTypeEnum.DateTime Then
                    objTextBox.Value = CDate(Parameter.DefaultValues(0)).ToShortDateString
                Else
                    objTextBox.Value = Parameter.DefaultValues(0)
                End If
            End If

            Return objTextBox

        End Function

#End Region

#Region "Event Handlers"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' Page_Load runs when the control is loaded
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[ttr]	20.02.2005  created
        '''     [ttr]   21.02.2005  changed the way user credentials are set
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Private Sub Page_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

            Try

                Dim myReportService As New ReportService.ReportingService
                Dim reportParametersArray As ReportParameter() = Nothing
                Dim reportParameterValues As New ArrayList
                Dim objParameter As ReportService.ReportParameter
                Dim objParameterValue As ReportService.ParameterValue

                ' set report server url
                myReportService.Url = ReportServerURL

                'Set the report credentials. Check if we use impersonation to access the report.
                ' U --> User credentials are set via module settings
                ' I --> We use the settings from the portal. Depending on your web.config this could be:
                '       MASCHINE\ASPNET or
                '       the users networkname

                If CType(Me.Settings("reportuser"), String) = "U" Then
                    myReportService.Credentials = New System.Net.NetworkCredential(Me.ReportUser, Me.ReportUserPwd, Me.ReportDomain)
                Else
                    myReportService.Credentials = System.Net.CredentialCache.DefaultCredentials
                End If

                ' check if Report URL parameter is defined 
                If Not Request("Report") Is Nothing Then
                    ReportPath = Request("Report")
                End If

                ' check if Report Zoom parameter is defined
                If Not Request("Zoom") Is Nothing Then
                    ReportZoom = Request("Zoom")
                End If

                ' check if Report Format parameter is defined
                If Not Request("Format") Is Nothing Then
                    ReportFormat = Request("Format")
                End If

                'set the Report hidden parameter
                txtReport.Value = ReportPath

                ''' --------------------------------------------------------------------------------
                ''' Get the list of parameters. without specifing to render the report
                ''' (ForRendering set to false), valid values for these parameters will
                ''' not be populated 
                ''' --------------------------------------------------------------------------------
                reportParametersArray = myReportService.GetReportParameters(ReportPath, Nothing, False, Nothing, Nothing)

                'If there are parameters then we generate an array with values that should be populated from either
                '    - previous submission , 
                '    - URL parameters, 
                '    - default value defined inside the report
                If reportParametersArray.Length > 0 Then

                    'loop over each parameter and build the array of values 
                    For Each objParameter In reportParametersArray

                        objParameterValue = New ParameterValue
                        objParameterValue.Name = objParameter.Name
                        objParameterValue.Label = objParameter.Name

                        'if the report value is coming from URL or previous submission then it is part of the Request
                        If Not Request(objParameter.Name) Is Nothing Then
                            'set the value from URL or submission
                            objParameterValue.Value = Request(objParameter.Name)
                        ElseIf Not objParameter.DefaultValues Is Nothing Then
                            'set the value to the one defined inside report
                            objParameterValue.Value = objParameter.DefaultValues(0)
                        Else
                            'in this case we dont have a value for this parameter
                            objParameterValue.Value = ""
                            'we cannot render this report
                            m_canRendered = False
                        End If

                        ' if parameter value is available add to list
                        If objParameterValue.Value <> "" Then
                            reportParameterValues.Add(objParameterValue)
                        End If

                        'check if this parameter is dependent of another one 
                        If Not (objParameter.Dependencies Is Nothing) Then
                            For Each d As String In objParameter.Dependencies
                                'add the parameter as a dependecy if was not already added
                                If Not m_Dependencies.Contains(d) Then
                                    m_Dependencies.Add(d, Nothing)
                                End If
                            Next
                        End If

                    Next 'from: For Each Parameter As ReportParameter In Parameters

                    ''' --------------------------------------------------------------------------------
                    ''' Get the list of parameters this time specifying to render the report 
                    ''' (ForRender set to true) to get valid values for these parameters 
                    ''' in the reportParametersArray 
                    ''' --------------------------------------------------------------------------------
                    reportParametersArray = myReportService.GetReportParameters(ReportPath, Nothing, True, reportParameterValues.ToArray(GetType(ParameterValue)), Nothing)

                    ' create html table with controls to select parameter values
                    BuildParameterTable(reportParametersArray, TablePrompts)

                End If 'from: If Parameters.Length > 0 Then

                ' declarations needed for further processing
                Dim TR As System.Web.UI.HtmlControls.HtmlTableRow
                Dim TD As System.Web.UI.HtmlControls.HtmlTableCell

                ''' --------------------------------------------------------------------------------
                ''' add table row with dropdowns for report zoom and report format
                ''' --------------------------------------------------------------------------------

                TR = New System.Web.UI.HtmlControls.HtmlTableRow

                ' add the zoom prompt
                TD = New System.Web.UI.HtmlControls.HtmlTableCell

                TD.InnerHtml = "Zoom:"
                TD.Attributes.Add("Class", "SubHead")
                TR.Controls.Add(TD)

                TD = New System.Web.UI.HtmlControls.HtmlTableCell
                TD.Controls.Add(GetDropDownZoom(Me.ReportZoom()))
                TR.Controls.Add(TD)

                ' add the format prompt
                TD = New System.Web.UI.HtmlControls.HtmlTableCell

                TD.InnerHtml = "Format:"
                TD.Attributes.Add("Class", "SubHead")
                TR.Controls.Add(TD)

                TD = New System.Web.UI.HtmlControls.HtmlTableCell
                TD.Controls.Add(GetDropDownFormat(Me.ReportFormat()))
                TR.Controls.Add(TD)

                TablePrompts.Controls.Add(TR)

                ''' --------------------------------------------------------------------------------
                ''' add table row with generate report button
                ''' --------------------------------------------------------------------------------

                ' add the submit button
                TR = New System.Web.UI.HtmlControls.HtmlTableRow
                TD = New System.Web.UI.HtmlControls.HtmlTableCell
                TD.ColSpan = 4
                TD.Align = "Center"
                TD.Attributes.Add("Class", "SubHead")

                Dim Button As System.Web.UI.HtmlControls.HtmlInputButton
                Button = New System.Web.UI.HtmlControls.HtmlInputButton("submit")
                Button.Value = "Generate Report"
                Button.EnableViewState = False

                TD.Controls.Add(Button)
                TR.Controls.Add(TD)

                TablePrompts.Controls.Add(TR)

                If m_canRendered Then

                    Dim deviceInfo As String
                    Dim streamRoot As String
                    streamRoot = "http://" & GetDomainName(Request) & "/getimage.aspx?report=" & ReportPath & "&amp;streamid="

                    Select Case ReportFormat()
                        Case "HTML4.0", "HTML3.2"
                            deviceInfo = "<DeviceInfo>"
                            deviceInfo &= "<StreamRoot>" & streamRoot & "</StreamRoot>"
                            deviceInfo &= "<Toolbar>False</Toolbar>"
                            deviceInfo &= "<Parameters>False</Parameters>"
                            deviceInfo &= "<HTMLFragment>True</HTMLFragment>"
                            deviceInfo &= "<StyleStream>False</StyleStream>"
                            deviceInfo &= "<Section>0</Section>"
                            deviceInfo &= "<Zoom>" & ReportZoom & "</Zoom>"
                            deviceInfo &= "</DeviceInfo>"
                        Case Else
                            deviceInfo = "<DeviceInfo></DeviceInfo>"
                    End Select


                    'SQLRS-ReportViewer

                    Dim credentials As DataSourceCredentials() = Nothing
                    Dim showHideToggle As String = Nothing
                    Dim encoding As String
                    Dim mimeType As String
                    Dim warnings As Warning() = Nothing
                    Dim reportHistoryParameters As ParameterValue() = Nothing
                    Dim historyID As String = Nothing
                    Dim streamIDs As String() = Nothing

                    Dim sh As New SessionHeader
                    myReportService.SessionHeaderValue = sh

                    Dim result As Byte() = Nothing

                    result = myReportService.Render(ReportPath, ReportFormat, historyID, deviceInfo, _
                                                    reportParameterValues.ToArray(GetType(ParameterValue)), _
                                                    credentials, showHideToggle, encoding, mimeType, _
                                                    reportHistoryParameters, warnings, streamIDs)

                    'store ReportingServices and parameters object is session layer in case we need it for image streams
                    Session("rs") = myReportService
                    Session("reportParameterValues") = reportParameterValues.ToArray(GetType(ParameterValue))

                    Select Case ReportFormat()
                        Case "HTML4.0", "HTML3.2"

                            Dim enc As System.Text.Encoding = System.Text.Encoding.UTF8

                            ' get the report as a string
                            Dim tmpReport As String = enc.GetString(result)
                            ' create url to navigate to this page
                            Dim strNewUrl As String = DotNetNuke.Common.Globals.NavigateURL()

                            'replace all occurrences of report server link with current page link
                            If InStr(strNewUrl, "?") = 0 Then
                                tmpReport = tmpReport.Replace(ReportServerURL.Replace("/ReportService.asmx", "?"), strNewUrl & "?Report=")
                            Else
                                tmpReport = tmpReport.Replace(ReportServerURL.Replace("/ReportService.asmx", "?"), strNewUrl & "&Report=")
                            End If

                            ReportPlaceholder.InnerHtml = tmpReport

                        Case Else

                            Response.ClearContent()
                            Response.AppendHeader("content-length", result.Length.ToString())
                            Response.ContentType = mimeType
                            Response.BinaryWrite(result)
                            Response.Flush()
                            Response.Close()

                    End Select

                End If 'from: If ReportCanBeRendered Then

            Catch exc As Exception
                ProcessModuleLoadException(Me, exc)
            End Try

        End Sub

#End Region

#Region "Web Form Designer Generated Code "

        'This call is required by the Web Form Designer.
        <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()

        End Sub

        Private Sub Page_Init(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Init
            'CODEGEN: This method call is required by the Web Form Designer
            'Do not modify it using the code editor.
            InitializeComponent()
        End Sub

#End Region

    End Class

End Namespace