#Region "Copyright and License"
'
' DbTom ReportingService Module
' Copyright (c) 2003 - 2005
' by Thomas Trefz (tt@dbresource.de)
'
' Download latest version at http://www.dbresource.de
' No registration required for download
'
' Parts of this module based on an article from Marian Dumitrascu at codeproject:
' http://www.codeproject.com/useritems/SQLRSViewer.asp
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
' documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
' the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
' to permit persons to whom the Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included in all copies or substantial portions 
' of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
' TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
' CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
' DEALINGS IN THE SOFTWARE.
'
#End Region

Imports System.Web.UI.WebControls
Imports DotNetNuke
Imports DotNetNuke.Common
Imports DotNetNuke.Services.Exceptions
Imports DotNetNuke.Services.Localization
Imports DotNetNuke.Common.Utilities
Imports ReportService

Namespace DbTom.ReportingService

    ''' -----------------------------------------------------------------------------
    ''' Namespace:  DbTom.ReportingService
    ''' Project:    DbTom Module Pack for DotNetNuke
    ''' Class:      Settings
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' The Settings ModuleSettingsBase is used to manage the 
    ''' settings for the ReportingService ReportViewer Module
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    '''     [ttr]   20.02.2005  created
    '''     [ttr]   21.02.2005  added settings for reportname, authentication mode
    '''                         and user credentials (name, password, domain)
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Public MustInherit Class Settings

        Inherits Entities.Modules.ModuleSettingsBase

#Region "Controls"

        Protected plReportServer As UI.UserControls.LabelControl
        Protected WithEvents txtReportServer As System.Web.UI.WebControls.TextBox
        Protected plReportName As UI.UserControls.LabelControl
        Protected WithEvents cboReportNames As System.Web.UI.WebControls.DropDownList
        Protected plReportUser As UI.UserControls.LabelControl
        Protected WithEvents optReportUser As System.Web.UI.WebControls.RadioButtonList
        Protected WithEvents rowReportUser As System.Web.UI.HtmlControls.HtmlTableRow
        Protected plReportUserName As UI.UserControls.LabelControl
        Protected WithEvents txtReportUserName As System.Web.UI.WebControls.TextBox
        Protected plReportUserPassword As UI.UserControls.LabelControl
        Protected WithEvents txtReportUserPassword As System.Web.UI.WebControls.TextBox
        Protected plReportUserDomain As UI.UserControls.LabelControl
        Protected WithEvents txtReportUserDomain As System.Web.UI.WebControls.TextBox

#End Region

#Region "Public Properties"

        Public Property ShowUserRow() As Boolean
            Get
                ShowUserRow = Me.rowReportUser.Visible
            End Get
            Set(ByVal Value As Boolean)
                Me.rowReportUser.Visible = Value
            End Set
        End Property

#End Region

#Region "Base Method Implementations"

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' LoadSettings loads the settings from the Database and displays them
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''     [ttr]   20.2.2005   created
        '''     [ttr]   21.2.2005   load dropdownlist with all reports available 
        '''                         for current user, user type selection, fields
        '''                         to enter username, password and domain
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub LoadSettings()

            Try

                If (Page.IsPostBack = False) Then

                    ' set url to reporting server
                    If CType(Me.TabModuleSettings("reportserverurl"), String) <> "" Then
                        Me.txtReportServer.Text = CType(Me.TabModuleSettings("reportserverurl"), String)
                    End If

                    ' load dropdownlist with available reports
                    Dim myReportService As New ReportService.ReportingService
                    Dim myItems As CatalogItem() = Nothing
                    Dim myItem As ReportService.CatalogItem

                    ' check if url to reporting server is available
                    If Me.txtReportServer.Text <> "" Then

                        ' get url to reporting server
                        myReportService.Url = Me.txtReportServer.Text
                        ' set user credentials to logon
                        myReportService.Credentials = System.Net.CredentialCache.DefaultCredentials
                        ' get a list of all children
                        myItems = myReportService.ListChildren("/", True)

                        ' find all reports, remove folders
                        For Each myItem In myItems
                            ' check if item is a report
                            If myItem.Type = ItemTypeEnum.Report Then
                                ' add item to the list
                                Me.cboReportNames.Items.Add(New ListItem(myItem.Path))
                            End If
                        Next

                        ' select current report
                        If Not Me.cboReportNames.Items.FindByText(CType(Me.Settings("reportpath"), String)) Is Nothing Then
                            Me.cboReportNames.Items.FindByText(CType(Me.Settings("reportpath"), String)).Selected = True
                        End If

                    Else
                        ' we should show a message for the user that he must enter a valid url
                    End If

                    ' set user credentials for reporting service
                    Me.optReportUser.Items.Add(New ListItem(Localization.GetString("optReportUser.Impersonation", LocalResourceFile), "I"))
                    Me.optReportUser.Items.Add(New ListItem(Localization.GetString("optReportUser.Username", LocalResourceFile), "U"))

                    If CType(Me.Settings("reportuser"), String) <> "" Then
                        Me.optReportUser.Items.FindByValue(CType(Me.Settings("reportuser"), String)).Selected = True
                        If CType(Me.Settings("reportuser"), String) = "U" Then
                            Me.rowReportUser.Visible = True
                        Else
                            Me.rowReportUser.Visible = False
                        End If
                    Else
                        Me.optReportUser.Items(0).Selected = True
                    End If

                    Me.txtReportUserName.Text = CType(Me.Settings("reportusername"), String)
                    Me.txtReportUserPassword.Text = CType(Me.Settings("reportuserpassword"), String)
                    Me.txtReportUserDomain.Text = CType(Me.Settings("reportuserdomain"), String)

                End If

            Catch exc As Exception    'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

        ''' -----------------------------------------------------------------------------
        ''' <summary>
        ''' UpdateSettings saves the modified settings to the Database
        ''' </summary>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        '''     [ttr]   20.02.2005  created
        '''     [ttr]   21.02.2005  support fo reportname, authentication mode and
        '''                         user credentials (name, password, domain)
        ''' </history>
        ''' -----------------------------------------------------------------------------
        Public Overrides Sub UpdateSettings()
            Try
                Dim objModules As New Entities.Modules.ModuleController

                objModules.UpdateTabModuleSetting(Me.TabModuleId, "reportserverurl", Me.txtReportServer.Text)

                If Me.cboReportNames.Items.Count > 0 Then
                    objModules.UpdateTabModuleSetting(Me.TabModuleId, "reportpath", Me.cboReportNames.SelectedItem.Text)
                End If

                objModules.UpdateTabModuleSetting(Me.TabModuleId, "reportuser", Me.optReportUser.SelectedItem.Value)
                objModules.UpdateTabModuleSetting(Me.TabModuleId, "reportusername", Me.txtReportUserName.Text)
                objModules.UpdateTabModuleSetting(Me.TabModuleId, "reportuserpassword", Me.txtReportUserPassword.Text)
                objModules.UpdateTabModuleSetting(Me.TabModuleId, "reportuserdomain", Me.txtReportUserDomain.Text)

            Catch exc As Exception    'Module failed to load
                ProcessModuleLoadException(Me, exc)
            End Try
        End Sub

#End Region

#Region "Event Handlers"

        Private Sub optReportUser_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles optReportUser.SelectedIndexChanged

            If optReportUser.SelectedItem.Value = "U" Then
                Me.ShowUserRow = True
            End If

        End Sub

#End Region

#Region " Web Form Designer Generated Code "

        'This call is required by the Web Form Designer.
        <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()

        End Sub
        'NOTE: The following placeholder declaration is required by the Web Form Designer.
        'Do not delete or move it.
        Private designerPlaceholderDeclaration As System.Object

        Private Sub Page_Init(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Init
            'CODEGEN: This method call is required by the Web Form Designer
            'Do not modify it using the code editor.
            InitializeComponent()
        End Sub

#End Region

    End Class

End Namespace
